@@ -1,21 +1,21 @@
-/*  Zutils - Utilities dealing with compressed files
-    Copyright (C) 2009-2019 Antonio Diaz Diaz.
+/* Zutils - Utilities dealing with compressed files
+   Copyright (C) 2009-2020 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-enum { fmt_bz2, fmt_gz, fmt_lz, fmt_xz, num_formats };
+enum { fmt_bz2, fmt_gz, fmt_lz, fmt_xz, num_formats };	// format_index
 const char * const format_names[num_formats] = { "bz2", "gz", "lz", "xz" };
 const char * const simple_extensions[num_formats] =
   { ".bz2", ".gz", ".lz", ".xz" };
