@@ -1,9 +1,9 @@
 #! /bin/sh
 # check script for Zutils - Utilities dealing with compressed files
-# Copyright (C) 2009-2019 Antonio Diaz Diaz.
+# Copyright (C) 2009-2020 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
-# to copy, distribute and modify it.
+# to copy, distribute, and modify it.
 
 LC_ALL=C
 export LC_ALL
@@ -74,6 +74,10 @@ for i in ${extensions}; do
 		test_failed $LINENO $i
 done
 
+printf "LZIP\001-.............................." | "${ZCAT}" -N > /dev/null 2>&1
+[ $? = 1 ] || test_failed $LINENO
+printf "LZIPxxxxxx" | "${ZCAT}" -N > /dev/null || test_failed $LINENO
+printf "BZh9xxxxxx" | "${ZCAT}" -N > /dev/null || test_failed $LINENO
 "${ZCAT}" -N -v -s "${testdir}"/zcat_vs.dat > /dev/null || test_failed $LINENO
 "${ZCAT}" -N < in > copy || test_failed $LINENO
 cmp in copy || test_failed $LINENO
@@ -91,6 +95,8 @@ cmp in copy || test_failed $LINENO
 cmp in copy || test_failed $LINENO
 "${ZCAT}" -N lz_only > copy || test_failed $LINENO
 cmp in copy || test_failed $LINENO
+cat in.lz in in in in | "${ZCAT}" -N > copy || test_failed $LINENO	# tdata
+cmp in copy || test_failed $LINENO
 "${ZCAT}" -N in in.gz in.bz2 in.lz -- -in- -in-.lz > copy || test_failed $LINENO
 cmp in6 copy || test_failed $LINENO
 "${ZCAT}" -Nq in in.gz in.bz2 in.lz "${bad0_lz}" -- -in- -in-.lz > copy
@@ -105,6 +111,8 @@ cmp in6 copy || test_failed $LINENO
 "${ZCAT}" -N -R . > /dev/null || test_failed $LINENO
 "${ZCAT}" -N -R > /dev/null || test_failed $LINENO
 
+"${ZCAT}" -Nq "" < in.lz > /dev/null
+[ $? = 1 ] || test_failed $LINENO
 "${ZCAT}" -Nq --format=, in.lz
 [ $? = 1 ] || test_failed $LINENO
 "${ZCAT}" -Nq --format=,lz in.lz
@@ -132,11 +140,11 @@ printf "\ntesting    zcmp-%s..." "$2"
 for i in ${extensions}; do
 	"${ZCMP}" -N in.$i || test_failed $LINENO $i
 	"${ZCMP}" -N in in.$i || test_failed $LINENO $i
-	"${ZCMP}" -N -i 1kB:1000 -n 500 in6 in.$i || test_failed $LINENO $i
 	"${ZCMP}" -N in in.$i --force-format=,$i || test_failed $LINENO $i
 	"${ZCMP}" -N in.$i in || test_failed $LINENO $i
-	"${ZCMP}" -N -i 1KiB:1024 -n 50 in.$i in6 || test_failed $LINENO $i
 	"${ZCMP}" -N in.$i in --force-format=$i || test_failed $LINENO $i
+	"${ZCMP}" -N -i 1kB:1000 -n 500 in6 in.$i || test_failed $LINENO $i
+	"${ZCMP}" -N -i 1KiB:1024 -n 50 in.$i in6 || test_failed $LINENO $i
 done
 
 "${ZCMP}" -Nq in in6
@@ -153,9 +161,13 @@ done
 "${ZCMP}" -N -i 0:11 -n 100 in.tar pin.tar || test_failed $LINENO
 "${ZCMP}" -N -i 0:11 -n 1Ki in.tar pin.tar || test_failed $LINENO
 "${ZCMP}" -N -i 0:11 -n 10KiB in.tar pin.tar || test_failed $LINENO
-"${ZCMP}" -N - || test_failed $LINENO
+"${ZCMP}" -N - - || test_failed $LINENO
+"${ZCMP}" -Nq -
+[ $? = 2 ] || test_failed $LINENO
 "${ZCMP}" -N in in || test_failed $LINENO
 "${ZCMP}" -N in || test_failed $LINENO
+"${ZCMP}" -N --format=gz,bz2 in || test_failed $LINENO
+"${ZCMP}" -N --format=gz in || test_failed $LINENO
 "${ZCMP}" -N in.lz in.gz || test_failed $LINENO
 "${ZCMP}" -N --lz='lzip -q' in.lz in.gz || test_failed $LINENO
 "${ZCMP}" -N in.gz -- -in-.lz || test_failed $LINENO
@@ -166,12 +178,16 @@ done
 "${ZCMP}" -N -- -in-.lz in || test_failed $LINENO
 "${ZCMP}" -N -- -in- in || test_failed $LINENO
 "${ZCMP}" -N in -- -in- || test_failed $LINENO
-"${ZCMP}" -N lz_only.lz < in || test_failed $LINENO
 "${ZCMP}" -N in.lz - < in || test_failed $LINENO
 "${ZCMP}" -N - in.lz < in || test_failed $LINENO
 "${ZCMP}" -N in - < in.lz || test_failed $LINENO
 "${ZCMP}" -N - in < in.lz || test_failed $LINENO
-"${ZCMP}" -N -q --force-format=lz in.lz
+"${ZCMP}" -N lz_only.lz - < in || test_failed $LINENO
+"${ZCMP}" -Nq lz_only.lz
+[ $? = 2 ] || test_failed $LINENO
+"${ZCMP}" -Nq "" in
+[ $? = 2 ] || test_failed $LINENO
+"${ZCMP}" -Nq --force-format=lz in.lz
 [ $? = 2 ] || test_failed $LINENO
 "${ZCMP}" -Nq --force-format=lz in.gz in.lz
 r=$?
@@ -184,7 +200,7 @@ r=$?
 [ $? = 2 ] || test_failed $LINENO
 "${ZCMP}" -Nq -n -1 in in
 [ $? = 2 ] || test_failed $LINENO
-"${ZCMP}" -Nq -n 100BB in in
+"${ZCMP}" -N -q -n 100BB in in
 [ $? = 2 ] || test_failed $LINENO
 "${ZCMP}" -N --bad-option in in 2> /dev/null
 [ $? = 2 ] || test_failed $LINENO
@@ -195,16 +211,20 @@ printf "\ntesting   zdiff-%s..." "$2"
 for i in ${extensions}; do
 	"${ZDIFF}" -N in.$i > /dev/null || test_failed $LINENO $i
 	"${ZDIFF}" -N in in.$i > /dev/null || test_failed $LINENO $i
-	"${ZDIFF}" -N --force-format=,$i in in.$i > /dev/null ||
+	"${ZDIFF}" -N in in.$i --force-format=,$i > /dev/null ||
 		test_failed $LINENO $i
 	"${ZDIFF}" -N in.$i in > /dev/null || test_failed $LINENO $i
-	"${ZDIFF}" -N --force-format=$i, in.$i in > /dev/null ||
+	"${ZDIFF}" -N in.$i in --force-format=$i, > /dev/null ||
 		test_failed $LINENO $i
 done
 
-"${ZDIFF}" -N in in6 > /dev/null && test_failed $LINENO
+"${ZDIFF}" -N in in6 > /dev/null
+[ $? = 1 ] || test_failed $LINENO
+# GNU diff 3.0 returns 2 when binary files differ
 "${ZDIFF}" -N in.tar pin.tar > /dev/null && test_failed $LINENO
-"${ZDIFF}" -N - || test_failed $LINENO
+"${ZDIFF}" -N - - || test_failed $LINENO
+"${ZDIFF}" -N - 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
 "${ZDIFF}" -N in in || test_failed $LINENO
 "${ZDIFF}" -N in || test_failed $LINENO
 "${ZDIFF}" -N --format=gz,bz2 in || test_failed $LINENO
@@ -219,14 +239,18 @@ done
 "${ZDIFF}" -N -- -in-.lz in > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N -- -in- in > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N in -- -in- > /dev/null || test_failed $LINENO
-"${ZDIFF}" -N lz_only.lz < in > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N in.lz - < in > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N - in.lz < in > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N in - < in.lz > /dev/null || test_failed $LINENO
 "${ZDIFF}" -N - in < in.lz > /dev/null || test_failed $LINENO
+"${ZDIFF}" -N lz_only.lz - < in > /dev/null || test_failed $LINENO
+"${ZDIFF}" -N lz_only.lz 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
+"${ZDIFF}" -N "" in 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
 "${ZDIFF}" -N --bz2='-bzip2' in.bz2 2> /dev/null
 [ $? = 2 ] || test_failed $LINENO
-"${ZDIFF}" -N -q --force-format=bz2 in.bz2 2> /dev/null
+"${ZDIFF}" -Nq --force-format=bz2 in.bz2 2> /dev/null
 [ $? = 2 ] || test_failed $LINENO
 "${ZDIFF}" -N -q --force-format=,lz in.lz in.bz2 > /dev/null 2>&1
 r=$?
@@ -237,14 +261,14 @@ r=$?
 mkdir tmp2
 cat in > tmp2/a || framework_failure
 cat in.lz > tmp2/a.lz || framework_failure
-"${ZDIFF}" -N --format=bz2 tmp2/a < /dev/null > /dev/null
-[ $? = 1 ] || test_failed $LINENO
-"${ZDIFF}" -N --format=gz tmp2/a < /dev/null > /dev/null
-[ $? = 1 ] || test_failed $LINENO
-"${ZDIFF}" -N --format=lz tmp2/a.lz < /dev/null > /dev/null
-[ $? = 1 ] || test_failed $LINENO
-"${ZDIFF}" -N --format=lz tmp2/a < /dev/null || test_failed $LINENO
-"${ZDIFF}" -N --format=un tmp2/a.lz < /dev/null || test_failed $LINENO
+"${ZDIFF}" -N --format=bz2 tmp2/a 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
+"${ZDIFF}" -N --format=gz tmp2/a 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
+"${ZDIFF}" -N --format=lz tmp2/a.lz 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
+"${ZDIFF}" -N --format=lz tmp2/a || test_failed $LINENO
+"${ZDIFF}" -N --format=un tmp2/a.lz || test_failed $LINENO
 rm -r tmp2 || framework_failure
 
 
@@ -300,6 +324,8 @@ done
 
 "${ZGREP}" -N "GNU" .
 [ $? = 1 ] || test_failed $LINENO
+"${ZGREP}" -N "GNU" "" < in.lz 2> /dev/null
+[ $? = 2 ] || test_failed $LINENO
 "${ZGREP}" -N --bad-option 2> /dev/null
 [ $? = 2 ] || test_failed $LINENO
 "${ZGREP}" -N "GNU" -s nx_file
@@ -336,11 +362,13 @@ done
 "${ZTEST}" -Nq in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz
 [ $? = 2 ] || test_failed $LINENO
 lines=`"${ZTEST}" -N in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
-[ "${lines}" -eq 2 ] || test_failed $LINENO
+[ "${lines}" -eq 2 ] || test_failed $LINENO "${lines}"
 lines=`"${ZTEST}" -Nv in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
-[ "${lines}" -eq 5 ] || test_failed $LINENO
+[ "${lines}" -eq 6 ] || test_failed $LINENO "${lines}"
 "${ZTEST}" -Nq < in
 [ $? = 2 ] || test_failed $LINENO
+"${ZTEST}" -Nq "" < in.lz
+[ $? = 1 ] || test_failed $LINENO
 dd if=in.lz bs=1000 count=1 2> /dev/null | "${ZTEST}" -N -q
 [ $? = 2 ] || test_failed $LINENO
 "${ZTEST}" -Nq --force-format=lz in.bz2
@@ -353,6 +381,7 @@ dd if=in.lz bs=1000 count=1 2> /dev/null | "${ZTEST}" -N -q
 
 printf "\ntesting zupdate-%s..." "$2"
 
+"${ZUPDATE}" -N "" || test_failed $LINENO
 cat in.bz2 > a.bz2 || framework_failure
 cat in.gz > a.gz || framework_failure
 "${ZUPDATE}" -Nq --bz2=bad_command a.bz2
@@ -415,6 +444,24 @@ cat in.gz > a.gz || framework_failure
 [ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
+cat in.bz2 > a.tbz || framework_failure		# keep combined extensions
+cat in.bz2 > b.tbz2 || framework_failure
+cat in.gz > c.tgz || framework_failure
+"${ZUPDATE}" -N a.tbz b.tbz2 c.tgz || test_failed $LINENO
+[ ! -e a.tbz ] || test_failed $LINENO
+[ ! -e b.tbz2 ] || test_failed $LINENO
+[ ! -e c.tgz ] || test_failed $LINENO
+[ ! -e a ] || test_failed $LINENO
+[ ! -e b ] || test_failed $LINENO
+[ ! -e c ] || test_failed $LINENO
+[ ! -e a.lz ] || test_failed $LINENO
+[ ! -e b.lz ] || test_failed $LINENO
+[ ! -e c.lz ] || test_failed $LINENO
+[ -e a.tlz ] || test_failed $LINENO
+[ -e b.tlz ] || test_failed $LINENO
+[ -e c.tlz ] || test_failed $LINENO
+rm -f a.tlz b.tlz c.tlz || framework_failure
+
 cat in.bz2 > a.bz2 || framework_failure
 cat "${bad0_gz}" > b.gz || framework_failure
 cat in.gz > c.gz || framework_failure
@@ -434,6 +481,13 @@ cat in.bz2 > a.bz2 || framework_failure
 [ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
+cat in.gz > 'name with spaces.gz' || framework_failure
+"${ZUPDATE}" -N -1 -q 'name with spaces.gz' || test_failed $LINENO
+[ ! -e 'name with spaces.gz' ] || test_failed $LINENO
+[ -e 'name with spaces.lz' ] || test_failed $LINENO
+"${ZCMP}" -N in 'name with spaces.lz' || test_failed $LINENO
+rm -f 'name with spaces.lz' || framework_failure
+
 mkdir tmp2
 mkdir tmp2/tmp3
 cat in.bz2 > tmp2/tmp3/a.bz2 || framework_failure
